/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.BacklogManagerDto
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.MsgId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BacklogManager", side = ProtocolSide.CLIENT)
interface BacklogManagerServerApi {
  /**
   * Loads backlog for [bufferId], where the message id is >= [first] and < [last].
   * If [first] or [last] is unset, the list will be unbounded in that direction.
   *
   * If a [limit] is set, the list will be truncated to the newest N messages.
   *
   * If both [first] and [last] are set, and the list of messages is not truncated by [limit],
   * [additional] messages will be loaded before [last].
   */
  @RpcCall("requestBacklog")
  suspend fun requestBacklog(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0
  )

  /**
   * Loads backlog for [bufferId], where the message id is >= [first] and < [last].
   * If [first] or [last] is unset, the list will be unbounded in that direction.
   *
   * If a [limit] is set, the list will be truncated to the newest N messages.
   *
   * If both [first] and [last] are set, and the list of messages is not truncated by [limit],
   * [additional] messages will be loaded before [last].
   *
   * Only messages matching [type] and [flags] will be returned and counted.
   */
  @RpcCall("requestBacklogFiltered")
  suspend fun requestBacklogFiltered(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1
  )

  /**
   * Loads backlog for [bufferId], where the message id is >= [first] and < [last].
   * If [first] or [last] is unset, the list will be unbounded in that direction.
   *
   * If a [limit] is set, the list will be truncated to the oldest N messages.
   *
   * Only messages matching [type] and [flags] will be returned and counted.
   */
  @RpcCall("requestBacklogForward")
  suspend fun requestBacklogForward(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1
  )

  /**
   * Loads backlog for all buffers, where the message id is >= [first] and < [last].
   * If [first] or [last] is unset, the list will be unbounded in that direction.
   *
   * If a [limit] is set, the list will be truncated to the newest N messages.
   *
   * If both [first] and [last] are set, and the list of messages is not truncated by [limit],
   * [additional] messages will be loaded before [last].
   */
  @RpcCall("requestBacklogAll")
  suspend fun requestBacklogAll(
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0
  )

  /**
   * Loads backlog for all buffers, where the message id is >= [first] and < [last].
   * If [first] or [last] is unset, the list will be unbounded in that direction.
   *
   * If a [limit] is set, the list will be truncated to the newest N messages.
   *
   * If both [first] and [last] are set, and the list of messages is not truncated by [limit],
   * [additional] messages will be loaded before [last].
   *
   * Only messages matching [type] and [flags] will be returned and counted.
   */
  @RpcCall("requestBacklogAllFiltered")
  suspend fun requestBacklogAllFiltered(
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1
  )

  @RpcCall("requestUpdate")
  suspend fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun requestUpdate(properties: BacklogManagerDto) = requestUpdate(properties.let(BacklogManagerDto.Serializer::serialize))
}
